/*
 * File   :     split.java
 * Author :     Tis Veugen
 * Date   :     1998-11-18
 * Version:     1.0
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class split extends Frame
   implements ActionListener, ItemListener, TextListener
{
  private static final int      SPLIT_0512__B = 512;
  private static final int      SPLIT_0001_KB = 1024;
  private static final int      SPLIT_0002_KB = 2*1024;
  private static final int      SPLIT_0004_KB = 4*1024;
  private static final int      SPLIT_0008_KB = 8*1024;
  private static final int      SPLIT_0016_KB = 16*1024;
  private static final int      SPLIT_0720_KB = 700*1024;
  private static final int      SPLIT_1440_KB = 1400*1024;
  private static final int      SPLIT_ALL_KB  = 1440*1024;

  private String []             maSizeStrings =
  {
    "1.4 MB",
    "720 KB",
    " 16 KB",
    "  8 KB",
    "  4 KB",
    "  2 KB",
    "  1 KB",
    "512 B " };
  private int []               maSizeValues =
  {
    SPLIT_1440_KB,
    SPLIT_0720_KB,
    SPLIT_0016_KB,
    SPLIT_0008_KB,
    SPLIT_0004_KB,
    SPLIT_0002_KB,
    SPLIT_0001_KB,
    SPLIT_0512__B };

  private Button                mInput;
  private Button                mOutput;
  private Button                mSplit;
  private Button                mMerge;

  private Choice                mSizeChoice;

  private TextField             mInputTextField;
  private TextField             mOutputTextField;

  private BufferedInputStream   mInputFile;
  private BufferedOutputStream  mOutputFile;

  private String                mDirectory = "";
  private int                   mChunkSize = SPLIT_1440_KB;
  private byte []               mBuffer;

  public                        split()
  {
    mInputFile  = null;
    mOutputFile = null;

    mBuffer = new byte[ SPLIT_1440_KB ];

    setTitle( "Splitter (v1.0)" );

    GridBagLayout               lGbl = new GridBagLayout();
    setLayout( lGbl );

    mInput  = addButton( "Input" );
    mOutput = addButton( "Output" );
    mSplit  = addButton( "Split" );
    mMerge  = addButton( "Merge" );

    mInputTextField  = addTextField();
    mOutputTextField = addTextField();

    mSizeChoice = addChoice();
    for ( int i = 0; i < 8; i++ )
    {
      mSizeChoice.add( maSizeStrings[ i ] );
    }

    GridBagConstraints          lGbc = new GridBagConstraints();

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.HORIZONTAL;
    lGbc.weightx = 20;
    lGbc.weighty = 10;

    add( mInput,   lGbc, 0, 0, 1, 1 );
    add( mOutput,  lGbc, 0, 1, 1, 1 );

    add( mSizeChoice, lGbc, 0, 2, 1, 1 );

    lGbc.fill = GridBagConstraints.NONE;
    add( mInputTextField,  lGbc, 1, 0, 3, 1 );
    add( mOutputTextField, lGbc, 1, 1, 3, 1 );
    add( mSplit, lGbc, 2, 2, 1, 1 );
    add( mMerge, lGbc, 3, 2, 1, 1 );

    addWindowListener( new WindowAdapter()
                       {
                         public void windowClosing(
                         WindowEvent evt )
                         {
                           setVisible( false );
                           System.exit( 0 );
                         }
                       }
                     );

    setLocation( 100, 100 );
    setSize( 400, 100 );
}

  public void                   add(
  Component                     oComponent,
  GridBagConstraints            oGbc,
  int                           x,
  int                           y,
  int                           w,
  int                           h )
  {
    oGbc.gridx = x;
    oGbc.gridy = y;
    oGbc.gridwidth  = w;
    oGbc.gridheight = h;
    add( oComponent, oGbc );
  }

  public Choice                 addChoice()
  {
    Choice                      lChoice = new Choice();

    lChoice.addItemListener( this );

    return lChoice;
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public TextField              addTextField()
  {
    TextField                   lTextField = new TextField( "", 40 );

    lTextField.addTextListener( this );

    return lTextField;
  }

  public void                   ActionInput()
  {
    String                      lFilename;
    String                      lExtension;

    lFilename = SetFile( "*.*", "Input file", FileDialog.LOAD );
    if ( lFilename != null )
    {
      mInputTextField.setText( lFilename );
      mOutputTextField.setText( lFilename );
    }
  }

  private void                  CloseInputFile()
  {
    if ( mInputFile != null )
    {
      try
      {
        mInputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Input file" );
      }
    }

    mInputFile = null;
  }

  private void                  CloseOutputFile()
  {
    if ( mOutputFile != null )
    {
      try
      {
        mOutputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Output file" );
      }
    }

    mOutputFile = null;
  }

  private String                OpenInputFile(
  String                        oInputFilename )
  {
    if ( ( oInputFilename == null      )
      || ( oInputFilename.equals( "" ) ) )
    {
      return " No Input file";
    }
    try
    {
      mInputFile = new BufferedInputStream(
                   new FileInputStream( oInputFilename ) );
    }
    catch ( Exception e )
    {
      return "Can't open Input file " + oInputFilename;
    }

    return null;
  }

  private String                OpenOutputFile(
  String                        oOutputFilename )
  {
    if ( ( oOutputFilename == null      )
      || ( oOutputFilename.equals( "" ) ) )
    {
      return " No Output file";
    }

    try
    {
      mOutputFile = new BufferedOutputStream(
                    new FileOutputStream( oOutputFilename ) );
    }
    catch ( Exception e )
    {
      return "Can't open Output file " + oOutputFilename;
    }

    return null;
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( this      // mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }

  private void                  PrintDialog(
  boolean                       bDialog,
  Frame                         oFrame,
  String                        oHeader,
  String                        oText )
  {
    if ( bDialog )
    {
      new PbnDialog( oFrame, oHeader, oText );
    }
    else
    {
      System.out.println( oText );
    }
  }

  private boolean               ExecuteSplit(
  String                        oInputFilename,
  String                        oOutputFilename,
  BufferedInputStream           oInputFile,
  int                           oChunkSize,
  boolean                       bDialog )
  {
    int                         lFileSize;
    int                         lIndex = 0;
    int                         lNrRead;
    String                      lString;

    try
    {
      lFileSize = oInputFile.available();
    }
    catch ( Exception e )
    {
      lString = "Can't get size of Input file " + oInputFilename;
      PrintDialog( bDialog, this, "Files", lString );
      return false;
    }

    for ( int lWritten = 0; lWritten < lFileSize; lWritten += oChunkSize )
    {
      try
      {
        lNrRead = mInputFile.read( mBuffer, 0, oChunkSize );
      }
      catch ( Exception e )
      {
        lString = "Can't read from Input file " + oInputFilename;
        PrintDialog( bDialog, this, "Files", lString );
        return false;
      }

      lIndex++;
      lString = OpenOutputFile( oOutputFilename + lIndex );
      if ( lString == null )
      {
        try
        {
          mOutputFile.write( mBuffer, 0, lNrRead );
        }
        catch ( Exception e )
        {
          lString = "Can't write to Output file " + oOutputFilename + lIndex;
        }
      }

      CloseOutputFile();
      if ( lString != null )
      {
        PrintDialog( bDialog, this, "Files", lString );
        return false;
      }
    }

    return true;
  }

  private boolean               ExecuteMerge(
  String                        oInputFilename,
  String                        oOutputFilename,
  BufferedOutputStream          oOutputFile,
  boolean                       bDialog )
  {
    String                      lString = null;
    int                         lIndex = 1;
    int                         lNrRead;

    while ( OpenInputFile( oInputFilename + lIndex ) == null )
    {
      lNrRead = 0;
      while ( lNrRead != -1 )
      {
        try
        {
          lNrRead = mInputFile.read( mBuffer, 0, SPLIT_ALL_KB );
        }
        catch ( Exception e )
        {
          lNrRead = -1;
        }

        if ( lNrRead > 0 )
        {
          try
          {
            mOutputFile.write( mBuffer, 0, lNrRead );
          }
          catch ( Exception e )
          {
            lString = "Can't write to Output file " + oOutputFilename;
            break;
          }
        }
      }
      CloseInputFile();

      if ( lString != null )
      {
        PrintDialog( bDialog, this, "Files", lString );
        return false;
      }

      lIndex++;
    }

    return true;
  }

  private String                StripExtension(
  String                        oFilename )
  {
    int                         lIndex;

    lIndex = oFilename.lastIndexOf( "." );
    if ( lIndex == -1 )
    {
      return oFilename + ".";
    }

    return oFilename.substring( 0, lIndex+1 );
  }

  private boolean               ActionConvert(
  String                        oInputFilename,
  String                        oOutputFilename,
  boolean                       bSplit,
  boolean                       bDialog )
  {
    String                      lString;
    boolean                     bOK = false;

    if ( bDialog )
    {
      this.setCursor( WAIT_CURSOR );
    }

    if ( bSplit )
    {
      lString = OpenInputFile( oInputFilename );
      if ( lString != null )
      {
        PrintDialog( bDialog, this, "Files", lString );
      }
      else
      {
        oOutputFilename = StripExtension( oOutputFilename );
        bOK = ExecuteSplit( oInputFilename
                          , oOutputFilename
                          , mInputFile
                          , mChunkSize
                          , bDialog );
        CloseInputFile();
      }
    }
    else
    {
      lString = OpenOutputFile( oOutputFilename );
      if ( lString != null )
      {
        PrintDialog( bDialog, this, "Files", lString );
      }
      else
      {
        oInputFilename = StripExtension( oInputFilename );
        bOK = ExecuteMerge( oInputFilename
                          , oOutputFilename
                          , mOutputFile
                          , bDialog );
        CloseOutputFile();
      }
    }

    if ( bDialog )
    {
      this.setCursor( DEFAULT_CURSOR );
    }

    return bOK;
  }

  private void                  ActionConverts(
  String                        oInputFilename,
  boolean                       bSplit,
  boolean                       bDialog )
  {
    File                        lFile = new File( oInputFilename );
    String                      lPattern = lFile.getName();
    String                      lPath;
    File                        lDirectory;
    String []                   lFiles;
    String []                   lMatchedFiles;
    boolean                     bOK;

    try
    {
      lPath = lFile.getParent();
      lDirectory = new File( lPath );
      lFiles = lDirectory.list();
      if ( lFiles == null )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "No files" );

        return;
      }
    }
    catch ( Exception e )
    {
      System.err.println( "Can't construct directory" );
      return;
    }

    lMatchedFiles = PatternMatch.MatchList( lFiles, lPattern );

    for ( int i = 0; i < lMatchedFiles.length; i++ )
    {
      File lConvertFile = new File( lDirectory, lMatchedFiles[ i ] );

      bOK = ActionConvert( lConvertFile.toString()
                         , lConvertFile.toString()
                         , bSplit
                         , bDialog );

      if ( ! bOK )
      {
        PrintDialog( bDialog, this, "Conversion error"
                   , "Can't execute conversion of file " + lMatchedFiles[ i ] );
      }
    }
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();
    String                      lFilename;

    if ( lName.equals( "Input" ) )
    {
      ActionInput();
    }
    else
    if ( lName.equals( "Output" ) )
    {
      lFilename = SetFile( "*.*", "Output file", FileDialog.LOAD );
      if ( lFilename != null )
      {
        mOutputTextField.setText( lFilename );
      }
    }
    else
    if ( ( lName.equals( "Split"   ) )
      || ( lName.equals( "Merge" ) ) )
    {
      String                    lInputFilename;
      lInputFilename = mInputTextField.getText();

      boolean                   bSplit = lName.equals( "Split" );
      boolean                   bOK;

      if ( ( lInputFilename.indexOf( "*" ) == -1 )
        && ( lInputFilename.indexOf( "?" ) == -1 ) )
      {
        bOK = ActionConvert( lInputFilename
                           , mOutputTextField.getText()
                           , bSplit
                           , true );
        if ( ! bOK )
        {
          PrintDialog( true, this, "File error", "Can't execute conversion" );
        }
        else
        {
          PrintDialog( true, this, "Conversion Result", "Ready" );
        }
      }
      else
      {
        ActionConverts( lInputFilename
                      , bSplit
                      , true );
        PrintDialog( true, this, "Conversion Result", "Ready" );
      }
    }
  }

  public void                   itemStateChanged(
  ItemEvent                     evt )
  {
    if ( evt.getStateChange() == ItemEvent.DESELECTED )
    {
      return;
    }

    String                      lItem = (String) evt.getItem();

    for ( int i = 0; i < 8; i++ )
    {
      if ( lItem.equals( maSizeStrings[ i ] ) )
      {
        mChunkSize = maSizeValues[ i ];
        break;
      }
    }
  }

  public void                   textValueChanged(
  TextEvent                     evt )
  {
  }

  public static void            main(
  String []                     args )
  {
    Frame                       lSplit;

    lSplit = new split();
    lSplit.show();
  }
}

class PbnDialog extends Dialog
{
  public                        PbnDialog(
  Frame                         oParent,
  String                        oHeader,
  String                        oText )
  {
    super( oParent, oHeader, true );

    Panel                       lPanel = new Panel();
    int                         NrLines;
    int                         Index;

    for ( NrLines = 0; ; NrLines++ )
    {
      Index = oText.indexOf( "\n" );
      if ( Index < 0 )
      {
        lPanel.add( new Label( oText ) );
        break;
      }
      lPanel.add( new Label( oText.substring( 0, Index ) ) );
      oText = oText.substring( Index+1 );
    }
    add( lPanel, "Center" );

    Button                      mOk = new Button( "OK" );
    lPanel = new Panel();
    lPanel.add( mOk );
    add( lPanel, "South" );

    mOk.addActionListener( new ActionListener()
                           {
                             public void actionPerformed(
                             ActionEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

        addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(
                             WindowEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

    setLocation( 150, 150 );
    setSize( 300, 120 + NrLines * 30 );
    show();
  }
}
